# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.azclierror import (
    CLIInternalError
)


def should_load_source(source):
    '''Check whether to load `az {source} connection`
    If {source} is an extension (e.g, spring-cloud), load the command group only when {source} is installed
    :param source: the source resource type
    '''
    from azure.cli.core.extension.operations import list_extensions
    from ._resource_config import SOURCE_RESOURCES_IN_EXTENSION

    # names of CLI installed extensions
    installed_extensions = [item.get('name') for item in list_extensions()]
    # if source resource is released as an extension, load our command groups
    # only when the extension is installed
    if source not in SOURCE_RESOURCES_IN_EXTENSION or source.value in installed_extensions:
        return True
    return False


def generate_random_string(length=5, prefix='', lower_only=False, ensure_complexity=False):
    '''Generate a random string
    :param length: the length of generated random string, not including the prefix
    :param prefix: the prefix string
    :param lower_only: ensure the generated string only includes lower case characters
    :param ensure_complexity: ensure the generated string satisfy complexity requirements
    '''
    import random
    import string

    if lower_only and ensure_complexity:
        raise CLIInternalError('lower_only and ensure_complexity can not both be specified to True')
    if ensure_complexity and length < 8:
        raise CLIInternalError('ensure_complexity needs length >= 8')

    character_set = string.ascii_letters + string.digits
    if lower_only:
        character_set = string.ascii_lowercase

    while True:
        randstr = '{}{}'.format(prefix, ''.join(random.sample(character_set, length)))
        lowers = [c for c in randstr if c.islower()]
        uppers = [c for c in randstr if c.isupper()]
        numbers = [c for c in randstr if c.isnumeric()]
        if not ensure_complexity or (lowers and uppers and numbers):
            break

    return randstr


def run_cli_cmd(cmd, retry=0):
    '''Run a CLI command
    :param cmd: The CLI command to be executed
    :param retry: The times to re-try
    '''
    import json
    import subprocess

    output = subprocess.run(cmd, shell=True, check=False, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
    if output.returncode != 0:
        if retry:
            run_cli_cmd(cmd, retry - 1)
        else:
            raise CLIInternalError('Command execution failed, command is: '
                                   '{}, error message is: {}'.format(cmd, output.stderr))

    return json.loads(output.stdout) if output.stdout else None



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-lines
# pylint: disable=unused-argument

from azure.cli.core.util import sdk_no_wait
from azure.mgmt.databoxedge.models import Sku


def databoxedge_device_list(client,
                            resource_group_name=None,
                            expand=None):
    if resource_group_name:
        return client.list_by_resource_group(resource_group_name=resource_group_name,
                                             expand=expand)
    return client.list_by_subscription(expand=expand)


def databoxedge_device_show(client,
                            device_name,
                            resource_group_name):
    return client.get(device_name=device_name,
                      resource_group_name=resource_group_name)


def databoxedge_device_create(client,
                              device_name,
                              resource_group_name,
                              location,
                              tags=None,
                              sku=None,
                              etag=None,
                              data_box_edge_device_status=None,
                              description=None,
                              model_description=None,
                              friendly_name=None,
                              no_wait=False):
    data_box_edge_device = {}
    data_box_edge_device['location'] = location
    if tags is not None:
        data_box_edge_device['tags'] = tags
    if sku is not None:
        data_box_edge_device['sku'] = Sku(name=sku)
    if etag is not None:
        data_box_edge_device['etag'] = etag
    if data_box_edge_device is not None:
        data_box_edge_device['data_box_edge_device_status'] = data_box_edge_device_status
    if description is not None:
        data_box_edge_device['description'] = description
    if model_description is not None:
        data_box_edge_device['model_description'] = model_description
    if friendly_name is not None:
        data_box_edge_device['friendly_name'] = friendly_name
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       device_name=device_name,
                       resource_group_name=resource_group_name,
                       data_box_edge_device=data_box_edge_device)


def databoxedge_device_update(client,
                              device_name,
                              resource_group_name,
                              tags=None):
    parameters = {}
    if tags is not None:
        parameters['tags'] = tags
    return client.update(device_name=device_name,
                         resource_group_name=resource_group_name,
                         parameters=parameters)


def databoxedge_device_delete(client,
                              device_name,
                              resource_group_name,
                              no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       device_name=device_name,
                       resource_group_name=resource_group_name)


def databoxedge_device_download_update(client,
                                       device_name,
                                       resource_group_name,
                                       no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_download_updates,
                       device_name=device_name,
                       resource_group_name=resource_group_name)


def databoxedge_device_install_update(client,
                                      device_name,
                                      resource_group_name,
                                      no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_install_updates,
                       device_name=device_name,
                       resource_group_name=resource_group_name)


def databoxedge_device_scan_for_update(client,
                                       device_name,
                                       resource_group_name,
                                       no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_scan_for_updates,
                       device_name=device_name,
                       resource_group_name=resource_group_name)


def databoxedge_device_show_update_summary(client,
                                           device_name,
                                           resource_group_name):
    return client.get_update_summary(device_name=device_name,
                                     resource_group_name=resource_group_name)


def databoxedge_alert_list(client,
                           device_name,
                           resource_group_name):
    return client.list_by_data_box_edge_device(device_name=device_name,
                                               resource_group_name=resource_group_name)


def databoxedge_alert_show(client,
                           device_name,
                           name,
                           resource_group_name):
    return client.get(device_name=device_name,
                      name=name,
                      resource_group_name=resource_group_name)


def databoxedge_bandwidth_schedule_list(client,
                                        device_name,
                                        resource_group_name):
    return client.list_by_data_box_edge_device(device_name=device_name,
                                               resource_group_name=resource_group_name)


def databoxedge_bandwidth_schedule_show(client,
                                        device_name,
                                        name,
                                        resource_group_name):
    return client.get(device_name=device_name,
                      name=name,
                      resource_group_name=resource_group_name)


def databoxedge_bandwidth_schedule_create(client,
                                          device_name,
                                          name,
                                          resource_group_name,
                                          start,
                                          stop,
                                          rate_in_mbps,
                                          days,
                                          no_wait=False):
    parameters = {}
    parameters['start'] = start
    parameters['stop'] = stop
    parameters['rate_in_mbps'] = rate_in_mbps
    parameters['days'] = days
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       device_name=device_name,
                       name=name,
                       resource_group_name=resource_group_name,
                       parameters=parameters)


def databoxedge_bandwidth_schedule_update(instance,
                                          device_name,
                                          name,
                                          resource_group_name,
                                          start,
                                          stop,
                                          rate_in_mbps,
                                          days,
                                          no_wait=False):
    if start is not None:
        instance.start = start
    if stop is not None:
        instance.stop = stop
    if rate_in_mbps is not None:
        instance.rate_in_mbps = rate_in_mbps
    if days is not None:
        instance.days = days
    return instance


def databoxedge_bandwidth_schedule_delete(client,
                                          device_name,
                                          name,
                                          resource_group_name,
                                          no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       device_name=device_name,
                       name=name,
                       resource_group_name=resource_group_name)


def databoxedge_show_job(client,
                         device_name,
                         name,
                         resource_group_name):
    return client.get(device_name=device_name,
                      name=name,
                      resource_group_name=resource_group_name)


def databoxedge_list_node(client,
                          device_name,
                          resource_group_name):
    return client.list_by_data_box_edge_device(device_name=device_name,
                                               resource_group_name=resource_group_name)


def databoxedge_order_list(client,
                           device_name,
                           resource_group_name):
    return client.list_by_data_box_edge_device(device_name=device_name,
                                               resource_group_name=resource_group_name)


def databoxedge_order_show(client,
                           device_name,
                           resource_group_name):
    return client.get(device_name=device_name,
                      resource_group_name=resource_group_name)


def databoxedge_order_create(client,
                             device_name,
                             resource_group_name,
                             address_line1,
                             postal_code,
                             city,
                             state,
                             country,
                             contact_person,
                             company_name,
                             phone,
                             email_list,
                             status,
                             comments=None,
                             address_line2=None,
                             address_line3=None,
                             no_wait=False):
    order = {}
    order['current_status'] = {}
    order['current_status']['status'] = status
    if comments is not None:
        order['current_status']['comments'] = comments
    order['shipping_address'] = {}
    order['shipping_address']['address_line1'] = address_line1
    if address_line2 is not None:
        order['shipping_address']['address_line2'] = address_line2
    if address_line3 is not None:
        order['shipping_address']['address_line3'] = address_line3
    order['shipping_address']['postal_code'] = postal_code
    order['shipping_address']['city'] = city
    order['shipping_address']['state'] = state
    order['shipping_address']['country'] = country
    order['contact_information'] = {}
    order['contact_information']['contact_person'] = contact_person
    order['contact_information']['company_name'] = company_name
    order['contact_information']['phone'] = phone
    order['contact_information']['email_list'] = email_list
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       device_name=device_name,
                       resource_group_name=resource_group_name,
                       order=order)


def databoxedge_order_update(instance,
                             device_name,
                             resource_group_name,
                             status=None,
                             comments=None,
                             address_line1=None,
                             address_line2=None,
                             address_line3=None,
                             postal_code=None,
                             city=None,
                             state=None,
                             country=None,
                             contact_person=None,
                             company_name=None,
                             phone=None,
                             email_list=None,
                             no_wait=False):
    if status is not None:
        instance.current_status.status = status
    if comments is not None:
        instance.current_status.comments = comments
    if address_line1 is not None:
        instance.shipping_address.address_line1 = address_line1
    if address_line2 is not None:
        instance.shipping_address.address_line2 = address_line2
    if address_line3 is not None:
        instance.shipping_address.address_line3 = address_line3
    if postal_code is not None:
        instance.shipping_address.postal_code = postal_code
    if city is not None:
        instance.shipping_address.city = city
    if state is not None:
        instance.shipping_address.state = state
    if country is not None:
        instance.shipping_address.country = country
    if contact_person is not None:
        instance.contact_information.contact_person = contact_person
    if company_name is not None:
        instance.contact_information.company_name = company_name
    if phone is not None:
        instance.contact_information.phone = phone
    if email_list is not None:
        instance.contact_information.email_list = email_list
    return instance


def databoxedge_order_delete(client,
                             device_name,
                             resource_group_name,
                             no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       device_name=device_name,
                       resource_group_name=resource_group_name)


def databoxedge_list_sku(client,
                         filter_=None):
    return client.list(filter=filter_)



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-lines

from knack.help_files import helps


helps['synapse'] = '''
    type: group
    short-summary: Manage Synapse
'''

helps['synapse kusto-operation'] = """
    type: group
    short-summary: Manage kusto operation with synapse
"""

helps['synapse kusto-operation list'] = """
    type: command
    short-summary: "Lists available operations for the Kusto sub-resources inside Microsoft.Synapse provider."
    examples:
      - name: KustoOperationsList
        text: |-
               az synapse kusto-operation list
"""

helps['synapse kusto pool'] = """
    type: group
    short-summary: Manage kusto pool with synapse
"""

helps['synapse kusto pool list-sku'] = """
    type: command
    short-summary: "Lists eligible SKUs for Kusto Pool resource."
    examples:
      - name: KustoPoolsListSkus
        text: |-
               az synapse kusto pool list-sku
"""

helps['synapse kusto pool'] = """
    type: group
    short-summary: Manage kusto pool with synapse
"""

helps['synapse kusto pool list'] = """
    type: command
    short-summary: "List all Kusto pools."
    examples:
      - name: List Kusto pools in a workspace
        text: |-
               az synapse kusto pool list --resource-group "kustorptest" --workspace-name "kustorptest"
"""

helps['synapse kusto pool show'] = """
    type: command
    short-summary: "Gets a Kusto pool."
    examples:
      - name: kustoPoolsGet
        text: |-
               az synapse kusto pool show --name "kustoclusterrptest4" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName"
"""

helps['synapse kusto pool delete'] = """
    type: command
    short-summary: "Deletes a Kusto pool."
    examples:
      - name: kustoPoolsDelete
        text: |-
               az synapse kusto pool delete --name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "kustorptest"
"""

helps['synapse kusto pool list-follower-database'] = """
    type: command
    short-summary: "Returns a list of databases that are owned by this Kusto Pool and were followed by another Kusto \
Pool."
    examples:
      - name: KustoPoolListFollowerDatabases
        text: |-
               az synapse kusto pool list-follower-database --name "kustoclusterrptest4" --resource-group \
"kustorptest" --workspace-name "kustorptest"
"""

helps['synapse kusto pool list-language-extension'] = """
    type: command
    short-summary: "Returns a list of language extensions that can run within KQL queries."
    examples:
      - name: KustoPoolListLanguageExtensions
        text: |-
               az synapse kusto pool list-language-extension --name "kustoclusterrptest4" --resource-group \
"kustorptest" --workspace-name "kustorptest"
"""

helps['synapse kusto pool list-sku'] = """
    type: command
    short-summary: "Returns the SKUs available for the provided resource."
    examples:
      - name: KustoPoolsListResourceSkus
        text: |-
               az synapse kusto pool list-sku --name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto pool start'] = """
    type: command
    short-summary: "Starts a Kusto pool."
    examples:
      - name: kustoPoolsStop
        text: |-
               az synapse kusto pool start --name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "kustorptest"
"""

helps['synapse kusto pool stop'] = """
    type: command
    short-summary: "Stops a Kusto pool."
    examples:
      - name: kustoPoolsStop
        text: |-
               az synapse kusto pool stop --name "kustoclusterrptest4" --resource-group "kustorptest" --workspace-name \
"kustorptest"
"""

helps['synapse kusto pool wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the synapse kusto pool is met.
    examples:
      - name: Pause executing next line of CLI script until the synapse kusto pool is successfully deleted.
        text: |-
               az synapse kusto pool wait --name "kustoclusterrptest4" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName" --deleted
      - name: Pause executing next line of CLI script until the synapse kusto pool is successfully created.
        text: |-
               az synapse kusto pool wait --name "kustoclusterrptest4" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName" --created
"""

helps['synapse kusto attached-database-configuration'] = """
    type: group
    short-summary: Manage kusto pool attached database configuration with synapse
"""

helps['synapse kusto attached-database-configuration list'] = """
    type: command
    short-summary: "Returns the list of attached database configurations of the given Kusto Pool."
    examples:
      - name: KustoPoolAttachedDatabaseConfigurationsListByKustoPool
        text: |-
               az synapse kusto attached-database-configuration list --kusto-pool-name "kustoclusterrptest4" \
--resource-group "kustorptest" --workspace-name "kustorptest"
"""

helps['synapse kusto attached-database-configuration show'] = """
    type: command
    short-summary: "Returns an attached database configuration."
    examples:
      - name: KustoPoolAttachedDatabaseConfigurationsGet
        text: |-
               az synapse kusto attached-database-configuration show --attached-database-configuration-name \
"attachedDatabaseConfigurations1" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "kustorptest"
"""

helps['synapse kusto attached-database-configuration create'] = """
    type: command
    short-summary: "Create an attached database configuration."
    parameters:
      - name: --table-level-sharing-properties --tlsp
        short-summary: "Table level sharing specifications"
        long-summary: |
            Usage: --table-level-sharing-properties tables-to-include=XX tables-to-exclude=XX \
external-tables-to-include=XX external-tables-to-exclude=XX materialized-views-to-include=XX \
materialized-views-to-exclude=XX

            tables-to-include: List of tables to include in the follower database
            tables-to-exclude: List of tables to exclude from the follower database
            external-tables-to-include: List of external tables to include in the follower database
            external-tables-to-exclude: List of external tables exclude from the follower database
            materialized-views-to-include: List of materialized views to include in the follower database
            materialized-views-to-exclude: List of materialized views exclude from the follower database
    examples:
      - name: KustoPoolAttachedDatabaseConfigurationsCreateOrUpdate
        text: |-
               az synapse kusto attached-database-configuration create --attached-database-configuration-name \
"attachedDatabaseConfigurations1" --kusto-pool-name "kustoclusterrptest4" --location "westus" --kusto-pool-resource-id \
"/subscriptions/12345678-1234-1234-1234-123456789098/resourceGroups/kustorptest/providers/Microsoft.Synapse/Workspaces/\
kustorptest/KustoPools/kustoclusterrptest4" --database-name "kustodatabase" --default-principals-modification-kind \
"Union" --table-level-sharing-properties external-tables-to-exclude="ExternalTable2" external-tables-to-include="Extern\
alTable1" materialized-views-to-exclude="MaterializedViewTable2" materialized-views-to-include="MaterializedViewTable1"\
 tables-to-exclude="Table2" tables-to-include="Table1" --resource-group "kustorptest" --workspace-name "kustorptest"
"""

helps['synapse kusto attached-database-configuration update'] = """
    type: command
    short-summary: "Update an attached database configuration."
    parameters:
      - name: --table-level-sharing-properties --tlsp
        short-summary: "Table level sharing specifications"
        long-summary: |
            Usage: --table-level-sharing-properties tables-to-include=XX tables-to-exclude=XX \
external-tables-to-include=XX external-tables-to-exclude=XX materialized-views-to-include=XX \
materialized-views-to-exclude=XX

            tables-to-include: List of tables to include in the follower database
            tables-to-exclude: List of tables to exclude from the follower database
            external-tables-to-include: List of external tables to include in the follower database
            external-tables-to-exclude: List of external tables exclude from the follower database
            materialized-views-to-include: List of materialized views to include in the follower database
            materialized-views-to-exclude: List of materialized views exclude from the follower database
"""

helps['synapse kusto attached-database-configuration delete'] = """
    type: command
    short-summary: "Deletes the attached database configuration with the given name."
    examples:
      - name: KustoPoolAttachedDatabaseConfigurationsDelete
        text: |-
               az synapse kusto attached-database-configuration delete --attached-database-configuration-name \
"attachedDatabaseConfigurations1" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "kustorptest"
"""

helps['synapse kusto attached-database-configuration wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the synapse kusto \
attached-database-configuration is met.
    examples:
      - name: Pause executing next line of CLI script until the synapse kusto attached-database-configuration is \
successfully created.
        text: |-
               az synapse kusto attached-database-configuration wait --attached-database-configuration-name \
"attachedDatabaseConfigurations1" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "kustorptest" --created
      - name: Pause executing next line of CLI script until the synapse kusto attached-database-configuration is \
successfully updated.
        text: |-
               az synapse kusto attached-database-configuration wait --attached-database-configuration-name \
"attachedDatabaseConfigurations1" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "kustorptest" --updated
      - name: Pause executing next line of CLI script until the synapse kusto attached-database-configuration is \
successfully deleted.
        text: |-
               az synapse kusto attached-database-configuration wait --attached-database-configuration-name \
"attachedDatabaseConfigurations1" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "kustorptest" --deleted
"""

helps['synapse kusto database'] = """
    type: group
    short-summary: Manage kusto pool database with synapse
"""

helps['synapse kusto database list'] = """
    type: command
    short-summary: "Returns the list of databases of the given Kusto pool."
    examples:
      - name: KustoDatabasesListByKustoPool
        text: |-
               az synapse kusto database list --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto database show'] = """
    type: command
    short-summary: "Returns a database."
    examples:
      - name: KustoPoolDatabasesGet
        text: |-
               az synapse kusto database show --database-name "KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" \
--resource-group "kustorptest" --workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto database create'] = """
    type: command
    short-summary: "Create a database."
    parameters:
      - name: --read-write-database
        short-summary: "Class representing a read write database."
        long-summary: |
            Usage: --read-write-database soft-delete-period=XX hot-cache-period=XX location=XX kind=XX

            soft-delete-period: The time the data should be kept before it stops being accessible to queries in \
TimeSpan.
            hot-cache-period: The time the data should be kept in cache for fast queries in TimeSpan.
            location: Resource location.
            kind: Required. Kind of the database
    examples:
      - name: KustoPoolDatabasesCreateOrUpdate
        text: |-
               az synapse kusto database create --database-name "KustoDatabase8" --kusto-pool-name \
"kustoclusterrptest4" --read-write-database location="westus" soft-delete-period="P1D" --resource-group "kustorptest" \
--workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto database update'] = """
    type: command
    short-summary: "Updates a database."
    parameters:
      - name: --read-write-database
        short-summary: "Class representing a read write database."
        long-summary: |
            Usage: --read-write-database soft-delete-period=XX hot-cache-period=XX location=XX kind=XX

            soft-delete-period: The time the data should be kept before it stops being accessible to queries in \
TimeSpan.
            hot-cache-period: The time the data should be kept in cache for fast queries in TimeSpan.
            location: Resource location.
            kind: Required. Kind of the database
    examples:
      - name: KustoPoolDatabasesUpdate
        text: |-
               az synapse kusto database update --database-name "KustoDatabase8" --kusto-pool-name \
"kustoclusterrptest4" --read-write-database soft-delete-period="P1D" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName"
"""

helps['synapse kusto database delete'] = """
    type: command
    short-summary: "Deletes the database with the given name."
    examples:
      - name: KustoPoolDatabasesDelete
        text: |-
               az synapse kusto database delete --database-name "KustoDatabase8" --kusto-pool-name \
"kustoclusterrptest4" --resource-group "kustorptest" --workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto database wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the synapse kusto database is met.
    examples:
      - name: Pause executing next line of CLI script until the synapse kusto database is successfully created.
        text: |-
               az synapse kusto database wait --database-name "KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" \
--resource-group "kustorptest" --workspace-name "synapseWorkspaceName" --created
      - name: Pause executing next line of CLI script until the synapse kusto database is successfully updated.
        text: |-
               az synapse kusto database wait --database-name "KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" \
--resource-group "kustorptest" --workspace-name "synapseWorkspaceName" --updated
      - name: Pause executing next line of CLI script until the synapse kusto database is successfully deleted.
        text: |-
               az synapse kusto database wait --database-name "KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" \
--resource-group "kustorptest" --workspace-name "synapseWorkspaceName" --deleted
"""

helps['synapse kusto data-connection'] = """
    type: group
    short-summary: Manage kusto pool data connection with synapse
"""

helps['synapse kusto data-connection list'] = """
    type: command
    short-summary: "Returns the list of data connections of the given Kusto pool database."
    examples:
      - name: KustoPoolDataConnectionsListByDatabase
        text: |-
               az synapse kusto data-connection list --database-name "KustoDatabase8" --kusto-pool-name \
"kustoclusterrptest4" --resource-group "kustorptest" --workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto data-connection show'] = """
    type: command
    short-summary: "Returns a data connection."
    examples:
      - name: KustoPoolDataConnectionsGet
        text: |-
               az synapse kusto data-connection show --data-connection-name "DataConnections8" --database-name \
"KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName"
"""

helps['synapse kusto data-connection event-grid'] = """
    type: group
    short-summary: Manage kusto pool data connection with synapse sub group event-grid
"""

helps['synapse kusto data-connection event-grid create'] = """
    type: command
    short-summary: "Create a data connection."
"""

helps['synapse kusto data-connection event-hub'] = """
    type: group
    short-summary: Manage kusto pool data connection with synapse sub group event-hub
"""

helps['synapse kusto data-connection event-hub create'] = """
    type: command
    short-summary: "Create a data connection."
    examples:
      - name: KustoPoolDataConnectionsCreateOrUpdate.json
        text: |-
               az synapse kusto data-connection event-hub create --data-connection-name "DataConnections8" \
--database-name "KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" --location "westus" --consumer-group \
"testConsumerGroup1" --event-hub-resource-id "/subscriptions/12345678-1234-1234-1234-123456789098/resourceGroups/kustor\
ptest/providers/Microsoft.EventHub/namespaces/eventhubTestns1/eventhubs/eventhubTest1" --resource-group "kustorptest" \
--workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto data-connection iot-hub'] = """
    type: group
    short-summary: Manage kusto pool data connection with synapse sub group iot-hub
"""

helps['synapse kusto data-connection iot-hub create'] = """
    type: command
    short-summary: "Create a data connection."
"""

helps['synapse kusto data-connection event-grid update'] = """
    type: command
    short-summary: "Updates a data connection."
"""

helps['synapse kusto data-connection event-hub update'] = """
    type: command
    short-summary: "Updates a data connection."
    examples:
      - name: KustoPoolDataConnectionsUpdate
        text: |-
               az synapse kusto data-connection event-hub update --data-connection-name "DataConnections8" \
--database-name "KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" --location "westus" --consumer-group \
"testConsumerGroup1" --event-hub-resource-id "/subscriptions/12345678-1234-1234-1234-123456789098/resourceGroups/kustor\
ptest/providers/Microsoft.EventHub/namespaces/eventhubTestns1/eventhubs/eventhubTest1" --resource-group "kustorptest" \
--workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto data-connection iot-hub update'] = """
    type: command
    short-summary: "Updates a data connection."
"""

helps['synapse kusto data-connection delete'] = """
    type: command
    short-summary: "Deletes the data connection with the given name."
    examples:
      - name: KustoPoolDataConnectionsDelete
        text: |-
               az synapse kusto data-connection delete --data-connection-name "kustoeventhubconnection1" \
--database-name "KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" \
--workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto data-connection wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the synapse kusto data-connection is met.
    examples:
      - name: Pause executing next line of CLI script until the synapse kusto data-connection is successfully created.
        text: |-
               az synapse kusto data-connection wait --data-connection-name "DataConnections8" --database-name \
"KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName" --created
      - name: Pause executing next line of CLI script until the synapse kusto data-connection is successfully deleted.
        text: |-
               az synapse kusto data-connection wait --data-connection-name "DataConnections8" --database-name \
"KustoDatabase8" --kusto-pool-name "kustoclusterrptest4" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName" --deleted
"""

helps['synapse kusto pool-principal-assignment'] = """
    type: group
    short-summary: Manage kusto pool principal assignment with synapse
"""

helps['synapse kusto pool-principal-assignment list'] = """
    type: command
    short-summary: "Lists all Kusto pool principalAssignments."
    examples:
      - name: KustoPoolPrincipalAssignmentsList
        text: |-
               az synapse kusto pool-principal-assignment list --kusto-pool-name "kustoclusterrptest4" \
--resource-group "kustorptest" --workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto pool-principal-assignment show'] = """
    type: command
    short-summary: "Gets a Kusto pool principalAssignment."
    examples:
      - name: KustoPoolPrincipalAssignmentsGet
        text: |-
               az synapse kusto pool-principal-assignment show --kusto-pool-name "kustoclusterrptest4" \
--principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto pool-principal-assignment create'] = """
    type: command
    short-summary: "Create a Kusto pool principalAssignment."
    examples:
      - name: KustoPoolPrincipalAssignmentsCreateOrUpdate
        text: |-
               az synapse kusto pool-principal-assignment create --kusto-pool-name "kustoclusterrptest4" \
--principal-id "87654321-1234-1234-1234-123456789123" --principal-type "App" --role "AllDatabasesAdmin" --tenant-id \
"12345678-1234-1234-1234-123456789123" --principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" \
--workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto pool-principal-assignment update'] = """
    type: command
    short-summary: "Update a Kusto pool principalAssignment."
"""

helps['synapse kusto pool-principal-assignment delete'] = """
    type: command
    short-summary: "Deletes a Kusto pool principalAssignment."
    examples:
      - name: KustoPoolPrincipalAssignmentsDelete
        text: |-
               az synapse kusto pool-principal-assignment delete --kusto-pool-name "kustoclusterrptest4" \
--principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto pool-principal-assignment wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the synapse kusto pool-principal-assignment \
is met.
    examples:
      - name: Pause executing next line of CLI script until the synapse kusto pool-principal-assignment is \
successfully created.
        text: |-
               az synapse kusto pool-principal-assignment wait --kusto-pool-name "kustoclusterrptest4" \
--principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name "synapseWorkspaceName" \
--created
      - name: Pause executing next line of CLI script until the synapse kusto pool-principal-assignment is \
successfully updated.
        text: |-
               az synapse kusto pool-principal-assignment wait --kusto-pool-name "kustoclusterrptest4" \
--principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name "synapseWorkspaceName" \
--updated
      - name: Pause executing next line of CLI script until the synapse kusto pool-principal-assignment is \
successfully deleted.
        text: |-
               az synapse kusto pool-principal-assignment wait --kusto-pool-name "kustoclusterrptest4" \
--principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name "synapseWorkspaceName" \
--deleted
"""

helps['synapse kusto database-principal-assignment'] = """
    type: group
    short-summary: Manage kusto pool database principal assignment with synapse
"""

helps['synapse kusto database-principal-assignment list'] = """
    type: command
    short-summary: "Lists all Kusto pool database principalAssignments."
    examples:
      - name: KustoPoolDatabasePrincipalAssignmentsList
        text: |-
               az synapse kusto database-principal-assignment list --database-name "Kustodatabase8" --kusto-pool-name \
"kustoclusterrptest4" --resource-group "kustorptest" --workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto database-principal-assignment show'] = """
    type: command
    short-summary: "Gets a Kusto pool database principalAssignment."
    examples:
      - name: KustoPoolDatabasePrincipalAssignmentsGet
        text: |-
               az synapse kusto database-principal-assignment show --database-name "Kustodatabase8" --kusto-pool-name \
"kustoclusterrptest4" --principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName"
"""

helps['synapse kusto database-principal-assignment create'] = """
    type: command
    short-summary: "Creates a Kusto pool database principalAssignment."
    examples:
      - name: KustoPoolDatabasePrincipalAssignmentsCreateOrUpdate
        text: |-
               az synapse kusto database-principal-assignment create --database-name "Kustodatabase8" \
--kusto-pool-name "kustoclusterrptest4" --principal-id "87654321-1234-1234-1234-123456789123" --principal-type "App" \
--role "Admin" --tenant-id "12345678-1234-1234-1234-123456789123" --principal-assignment-name "kustoprincipal1" \
--resource-group "kustorptest" --workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto database-principal-assignment update'] = """
    type: command
    short-summary: "Update a Kusto pool database principalAssignment."
"""

helps['synapse kusto database-principal-assignment delete'] = """
    type: command
    short-summary: "Deletes a Kusto pool principalAssignment."
    examples:
      - name: KustoPoolDatabasePrincipalAssignmentsDelete
        text: |-
               az synapse kusto database-principal-assignment delete --database-name "Kustodatabase8" \
--kusto-pool-name "kustoclusterrptest4" --principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" \
--workspace-name "synapseWorkspaceName"
"""

helps['synapse kusto database-principal-assignment wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the synapse kusto \
database-principal-assignment is met.
    examples:
      - name: Pause executing next line of CLI script until the synapse kusto database-principal-assignment is \
successfully created.
        text: |-
               az synapse kusto database-principal-assignment wait --database-name "Kustodatabase8" --kusto-pool-name \
"kustoclusterrptest4" --principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName" --created
      - name: Pause executing next line of CLI script until the synapse kusto database-principal-assignment is \
successfully updated.
        text: |-
               az synapse kusto database-principal-assignment wait --database-name "Kustodatabase8" --kusto-pool-name \
"kustoclusterrptest4" --principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName" --updated
      - name: Pause executing next line of CLI script until the synapse kusto database-principal-assignment is \
successfully deleted.
        text: |-
               az synapse kusto database-principal-assignment wait --database-name "Kustodatabase8" --kusto-pool-name \
"kustoclusterrptest4" --principal-assignment-name "kustoprincipal1" --resource-group "kustorptest" --workspace-name \
"synapseWorkspaceName" --deleted
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger
from knack.util import todict
from azure.cli.core.util import sdk_no_wait
from azure.cli.core.azclierror import (
    RequiredArgumentMissingError,
    ValidationError,
    AzureResponseError
)
from ._resource_config import (
    SUPPORTED_AUTH_TYPE,
    SUPPORTED_CLIENT_TYPE,
    TARGET_RESOURCES
)
from ._validators import (
    get_source_resource_name,
    get_target_resource_name
)
from ._addon_factory import AddonFactory
# pylint: disable=unused-argument


logger = get_logger(__name__)
err_msg = 'Required argument is missing, please provide the arguments: {}'


def connection_list(client,
                    source_resource_group=None,
                    source_id=None,
                    site=None,
                    # HACK, only for bugbash
                    # deployment=None,
                    spring=None, app=None):
    if not source_id:
        raise RequiredArgumentMissingError(err_msg.format('--source-id'))
    return client.list(resource_uri=source_id)


def connection_list_support_types(cmd, client,
                                  target_resource_type=None):
    results = []
    source = get_source_resource_name(cmd)

    targets = SUPPORTED_AUTH_TYPE.get(source).keys()
    if target_resource_type is not None:
        targets = []
        for resource in TARGET_RESOURCES:
            if target_resource_type == resource.value:
                targets.append(resource)
                break

    for target in targets:
        auth_types = SUPPORTED_AUTH_TYPE.get(source).get(target)
        client_types = SUPPORTED_CLIENT_TYPE.get(source).get(target)
        auth_types = [item.value for item in auth_types]
        client_types = [item.value for item in client_types]
        results.append({
            'source': source.value,
            'target': target.value,
            'auth_types': auth_types,
            'client_types': client_types
        })

    return results


def connection_show(client,
                    connection_name=None,
                    source_resource_group=None,
                    source_id=None,
                    indentifier=None,
                    site=None,
                    # HACK, only for bugbash
                    # deployment=None,
                    spring=None, app=None):
    if not source_id or not connection_name:
        raise RequiredArgumentMissingError(err_msg.format('--source-id, --connection'))
    return client.get(resource_uri=source_id,
                      linker_name=connection_name)


def connection_delete(client,
                      connection_name=None,
                      source_resource_group=None,
                      source_id=None,
                      indentifier=None,
                      site=None,
                      # HACK, only for bugbash
                      # deployment=None,
                      spring=None, app=None,
                      no_wait=False):
    if not source_id or not connection_name:
        raise RequiredArgumentMissingError(err_msg.format('--source-id, --connection'))

    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       resource_uri=source_id,
                       linker_name=connection_name)


def connection_list_configuration(client,
                                  connection_name=None,
                                  source_resource_group=None,
                                  source_id=None,
                                  indentifier=None,
                                  site=None,
                                  # HACK, only for bugbash
                                  # deployment=None
                                  spring=None, app=None):
    if not source_id or not connection_name:
        raise RequiredArgumentMissingError(err_msg.format('--source-id, --connection'))
    return client.list_configurations(resource_uri=source_id,
                                      linker_name=connection_name)


def connection_validate(client,
                        connection_name=None,
                        source_resource_group=None,
                        source_id=None,
                        indentifier=None,
                        site=None,
                        # HACK, only for bugbash
                        # deployment=None
                        spring=None, app=None):
    if not source_id or not connection_name:
        raise RequiredArgumentMissingError(err_msg.format('--source-id, --connection'))
    return client.begin_validate(resource_uri=source_id,
                                 linker_name=connection_name)


def connection_create(cmd, client,  # pylint: disable=too-many-locals
                      connection_name=None, client_type=None,
                      source_resource_group=None, source_id=None,
                      target_resource_group=None, target_id=None,
                      secret_auth_info=None, secret_auth_info_auto=None,
                      user_identity_auth_info=None, system_identity_auth_info=None,
                      service_principal_auth_info_secret=None,
                      new_addon=False, no_wait=False,
                      site=None,                                             # Resource.WebApp
                      # HACK, only for bugbash
                      # deployment=None
                      spring=None, app=None,                                 # Resource.SpringCloud
                      server=None, database=None,                            # Resource.*Postgres, Resource.*Sql*
                      vault=None,                                            # Resource.KeyVault
                      account=None, key_space=None, graph=None, table=None,  # Resource.Cosmos*, Resource.Storage*
                      config_store=None,                                     # Resource.AppConfig
                      namespace=None):                                       # Resource.EventHub

    if not source_id:
        raise RequiredArgumentMissingError(err_msg.format('--source-id'))
    if not new_addon and not target_id:
        raise RequiredArgumentMissingError(err_msg.format('--target-id'))

    all_auth_info = []
    if secret_auth_info is not None:
        all_auth_info.append(secret_auth_info)
    if secret_auth_info_auto is not None:
        all_auth_info.append(secret_auth_info_auto)
    if user_identity_auth_info is not None:
        all_auth_info.append(user_identity_auth_info)
    if system_identity_auth_info is not None:
        all_auth_info.append(system_identity_auth_info)
    if service_principal_auth_info_secret is not None:
        all_auth_info.append(service_principal_auth_info_secret)
    if not new_addon and len(all_auth_info) != 1:
        raise ValidationError('Only one auth info is needed')
    auth_info = all_auth_info[0] if len(all_auth_info) == 1 else None

    parameters = {
        'target_id': target_id,
        'auth_info': auth_info,
        'client_type': client_type
    }

    if new_addon:
        target_type = get_target_resource_name(cmd)
        addon = AddonFactory.get(target_type)(cmd, source_id)
        target_id, auth_info = addon.provision()
        parameters['target_id'] = target_id
        parameters['auth_info'] = auth_info
        logger.warning('Start creating the connection')

        try:
            linker = client.begin_create_or_update(resource_uri=source_id,
                                                   linker_name=connection_name,
                                                   parameters=parameters)
            return linker
        except Exception as e:
            logger.warning('Connection creation failed, start rolling back')
            addon.rollback()
            raise AzureResponseError('{}. Provision failed, please create the target resource '
                                     'manually and then create the connection.'.format(str(e)))

    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       resource_uri=source_id,
                       linker_name=connection_name,
                       parameters=parameters)


def connection_update(client,
                      connection_name=None, client_type=None,
                      source_resource_group=None, source_id=None, indentifier=None,
                      secret_auth_info=None, secret_auth_info_auto=None,
                      user_identity_auth_info=None, system_identity_auth_info=None,
                      service_principal_auth_info_secret=None,
                      no_wait=False,
                      site=None,                                              # Resource.WebApp
                      # HACK, only for bugbash
                      # deployment=None
                      spring=None, app=None):                                 # Resource.SpringCloud

    linker = todict(client.get(resource_uri=source_id, linker_name=connection_name))

    all_auth_info = []
    if secret_auth_info is not None:
        all_auth_info.append(secret_auth_info)
    if secret_auth_info_auto is not None:
        all_auth_info.append(secret_auth_info_auto)
    if user_identity_auth_info is not None:
        all_auth_info.append(user_identity_auth_info)
    if system_identity_auth_info is not None:
        all_auth_info.append(system_identity_auth_info)
    if service_principal_auth_info_secret is not None:
        all_auth_info.append(service_principal_auth_info_secret)

    # validate auth info
    if len(all_auth_info) > 1:
        raise ValidationError('Only one auth info is needed')
    # when user provides auth info
    if len(all_auth_info) == 1:
        auth_info = all_auth_info[0]
    # when user doesn't provide auth info and linker is not secret-with-username type
    elif not all_auth_info and (linker.get('authInfo').get('authType') != 'secret' or
                                not linker.get('authInfo').get('name')):
        auth_info = linker.get('authInfo')
    else:
        raise ValidationError('Auth info argument should be provided when '
                              'updating the connection: {}'.format(linker.get('name')))
    # validate the properties to be updated
    if client_type is None and not all_auth_info:
        raise ValidationError('Either client type or auth info should be specified to update')

    parameters = {
        'target_id': linker.get('targetId'),
        'auth_info': auth_info,
        'client_type': client_type or linker.get('clienType'),
    }

    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       resource_uri=source_id,
                       linker_name=connection_name,
                       parameters=parameters)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core._profile import Profile


def cf_connection_cl(cli_ctx, *_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.servicelinker import MicrosoftServiceLinker

    # pylint: disable=protected-access
    # Hack: set custom header to work around OBO
    profile = Profile(cli_ctx=cli_ctx)
    creds, _, _ = profile.get_raw_token()

    client = get_mgmt_service_client(cli_ctx, MicrosoftServiceLinker, subscription_bound=False)
    client._client._config.headers_policy._headers['x-ms-cupertino-test-token'] = creds[1]
    return client


def cf_linker(cli_ctx, *_):
    return cf_connection_cl(cli_ctx).linker



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.arguments import CLIArgumentType

from azure.cli.core.commands.parameters import (
    tags_type, get_resource_name_completion_list, resource_group_name_type, get_enum_type)

from azure.cli.command_modules.dls._validators import validate_resource_group_name
from azure.mgmt.datalake.store.models import (
    FirewallState,
    TrustedIdProviderState,
    TierType,
    FirewallAllowAzureIpsState)

from azure.mgmt.datalake.store.models import EncryptionConfigType


# pylint: disable=line-too-long, too-many-statements
def load_arguments(self, _):
    from ._validators import (
        validate_subnet
    )
    # ARGUMENT DEFINITIONS
    datalake_store_name_type = CLIArgumentType(help='Name of the Data Lake Store account.', options_list=['--account_name'], completer=get_resource_name_completion_list('Microsoft.DataLakeStore/accounts'), id_part='name')

    # PARAMETER REGISTRATIONS
    # global
    with self.argument_context('dls') as c:
        c.argument('account_name', datalake_store_name_type, options_list=['--account', '-n'])
        c.argument('top', help='Maximum number of items to return.', type=int)
        c.argument('skip', help='The number of items to skip over before returning elements.', type=int)
        c.argument('count', help='The Boolean value of true or false to request a count of the matching resources included with the resources in the response, e.g. Categories?$count=true.', type=bool)

    # global account
    with self.argument_context('dls account') as c:
        c.argument('tags', tags_type)
        c.argument('resource_group_name', resource_group_name_type, id_part=None, required=False, help='If not specified, will attempt to discover the resource group for the specified Data Lake Store account.', validator=validate_resource_group_name)
        c.argument('tier', arg_type=get_enum_type(TierType), help='The desired commitment tier for this account to use.')
        c.argument('default_group', help='Name of the default group to give permissions to for freshly created files and folders in the Data Lake Store account.')
        c.argument('key_version', help='Key version for the user-assigned encryption type.')

    # account
    for scope in ['dls account show', 'dls account delete']:
        with self.argument_context(scope) as c:
            c.argument('name', datalake_store_name_type, options_list=['--account', '-n'])

    with self.argument_context('dls account create') as c:
        c.argument('resource_group_name', resource_group_name_type, validator=None)
        c.argument('account_name', datalake_store_name_type, options_list=['--account', '-n'], completer=None)
        c.argument('encryption_type', arg_type=get_enum_type(EncryptionConfigType), help='Indicates what type of encryption to provision the account with. By default, encryption is ServiceManaged. If no encryption is desired, it must be explicitly set with the --disable-encryption flag.')
        c.argument('disable_encryption', help='Indicates that the account will not have any form of encryption applied to it.', action='store_true')

    with self.argument_context('dls account update') as c:
        c.argument('trusted_id_provider_state', arg_type=get_enum_type(TrustedIdProviderState), help='Enable/disable the existing trusted ID providers.')
        c.argument('firewall_state', arg_type=get_enum_type(FirewallState), help='Enable/disable existing firewall rules.')
        c.argument('allow_azure_ips', arg_type=get_enum_type(FirewallAllowAzureIpsState), help='Allow/block Azure originating IPs through the firewall')

    with self.argument_context('dls account list') as c:
        c.argument('resource_group_name', resource_group_name_type, validator=None)

    #####
    # virtual network rule
    #####
    with self.argument_context('dls account network-rule') as c:
        c.argument('account_name', datalake_store_name_type, options_list=['--account-name'], id_part=None)
        c.argument('resource_group_name', resource_group_name_type, id_part='resource_group')

        c.argument('virtual_network_rule_name',
                   options_list=['--name', '-n'],
                   help='The virtual network rule name',
                   id_part='name')

        c.argument('subnet',
                   options_list=['--subnet'],
                   help='Name or ID of the subnet that allows access to DLS. '
                   'If subnet name is provided, --name must be provided.')

    with self.argument_context('dls account network-rule create') as c:
        c.extra('vnet_name',
                help='The virtual network rule name',
                validator=validate_subnet)

    with self.argument_context('dls account network-rule update') as c:
        c.argument('subnet_id',
                   options_list=['--subnet'],
                   help='Name or ID of the subnet that allows access to DLS. '
                   'If subnet name is provided, --name must be provided.')

        c.extra('vnet_name', help='The virtual network rule name')

    with self.argument_context('dls account network-rule list') as c:
        c.argument('virtual_network_rule_name', id_part=None)

    # filesystem
    with self.argument_context('dls fs') as c:
        c.argument('path', help='The path in the specified Data Lake Store account where the action should take place. In the format \'/folder/file.txt\', where the first \'/\' after the DNS indicates the root of the file system.')
        c.argument('overwrite', help='Indicates that, if the destination file or folder exists, it should be overwritten', action='store_true')
        c.argument('chunk_size', help='Number of bytes for a chunk. Large files are split into chunks. Files smaller than this number will always be transferred in a single thread.', type=int, default=268435456, required=False)
        c.argument('buffer_size', help='Number of bytes for internal buffer. This block cannot be bigger than a chunk and cannot be smaller than a block.', type=int, default=4194304, required=False)
        c.argument('block_size', help='Number of bytes for a block. Within each chunk, we write a smaller block for each API call. This block cannot be bigger than a chunk.', type=int, default=4194304, required=False)
        c.ignore('progress_callback')

    with self.argument_context('dls fs create') as c:
        c.argument('force', help='Indicates that, if the file or folder exists, it should be overwritten', action='store_true')
        c.argument('folder', help='Indicates that this new item is a folder and not a file.', action='store_true')

    with self.argument_context('dls fs delete') as c:
        c.argument('recurse', help='Indicates this should be a recursive delete of the folder.', action='store_true')

    with self.argument_context('dls fs upload') as c:
        c.argument('thread_count', help='Specify the parallelism of the upload. Default is the number of cores in the local machine.', type=int)

    with self.argument_context('dls fs download') as c:
        c.argument('thread_count', help='Specify the parallelism of the download. Default is the number of cores in the local machine.', type=int)

    with self.argument_context('dls fs preview') as c:
        c.argument('force', help='Indicates that, if the preview is larger than 1MB, still retrieve it. This can potentially be very slow, depending on how large the file is.', action='store_true')

    with self.argument_context('dls fs join') as c:
        c.argument('force', help='Indicates that, if the destination file already exists, it should be overwritten', action='store_true')
        c.argument('source_paths', help='The list of files in the specified Data Lake Store account to join.', nargs='+')

    with self.argument_context('dls fs move') as c:
        c.argument('force', help='Indicates that, if the destination file or folder already exists, it should be overwritten and replaced with the file or folder being moved.', action='store_true')

    with self.argument_context('dls fs set-expiry') as c:
        c.argument('expiration_time', help='The absolute value of the expiration time expressed as milliseconds since the epoch.')

    # filesystem access params
    with self.argument_context('dls fs access') as c:
        c.argument('acl_spec', help=" The ACL specification to set on the path in the format '[default:]user|group|other:[entity id or UPN]:r|-w|-x|-,[default:]user|group|other:[entity id or UPN]:r|-w|-x|-,...'.")

    with self.argument_context('dls fs access set-permission') as c:
        c.argument('permission', help='The octal representation of the permissions for user, group and mask (for example: 777 is full rwx for all entities)', type=int)

    with self.argument_context('dls fs access remove-all') as c:
        c.argument('default_acl', help='A switch that, if specified, indicates that the remove ACL operation should remove the default ACL of the folder. Otherwise the regular ACL is removed.', action='store_true')



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-lines
# pylint: disable=too-many-statements

from azure.cli.core.commands.parameters import (
    get_three_state_flag,
    get_enum_type,
    resource_group_name_type,
    get_location_type
)
from azure.cli.core.commands.validators import get_default_location_from_resource_group
from ..action import (
    AddTableLevelSharingProperties,
    AddReadWriteDatabase
)


def load_arguments(self, _):

    with self.argument_context('synapse kusto pool list') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace')

    with self.argument_context('synapse kusto pool show') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', options_list=['--name', '-n', '--kusto-pool-name'], type=str, help='The name of '
                   'the Kusto pool.', id_part='child_name_1')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool delete') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('kusto_pool_name', options_list=['--name', '-n', '--kusto-pool-name'], type=str, help='The name of '
                   'the Kusto pool.', id_part='child_name_1')

    with self.argument_context('synapse kusto pool list-follower-database') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', options_list=['--name', '-n', '--kusto-pool-name'], type=str, help='The name of '
                   'the Kusto pool.')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool list-language-extension') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', options_list=['--name', '-n', '--kusto-pool-name'], type=str, help='The name of '
                   'the Kusto pool.')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool list-sku') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', options_list=['--name', '-n', '--kusto-pool-name'], type=str, help='The name of '
                   'the Kusto pool.')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool start') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', options_list=['--name', '-n', '--kusto-pool-name'], type=str, help='The name of '
                   'the Kusto pool.', id_part='child_name_1')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool stop') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', options_list=['--name', '-n', '--kusto-pool-name'], type=str, help='The name of '
                   'the Kusto pool.', id_part='child_name_1')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool wait') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', options_list=['--name', '-n', '--kusto-pool-name'], type=str, help='The name of '
                   'the Kusto pool.', id_part='child_name_1')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto attached-database-configuration list') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto attached-database-configuration show') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('attached_database_configuration_name', options_list=['--attached-database-configuration-name',
                                                                         '--adcn'], type=str, help='The name of the '
                   'attached database configuration.', id_part='child_name_2')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto attached-database-configuration create') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('attached_database_configuration_name', options_list=['--attached-database-configuration-name',
                                                                         '--adcn'], type=str, help='The name of the '
                   'attached database configuration.')
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('database_name', type=str, help='The name of the database which you would like to attach, use * if '
                   'you want to follow all current and future databases.')
        c.argument('kusto_pool_resource_id', type=str, help='The resource id of the kusto pool where the databases you '
                   'would like to attach reside.')
        c.argument('default_principals_modification_kind', options_list=['--default-principals-modification-kind',
                                                                         '--dpmk'], arg_type=get_enum_type(['Union',
                                                                                                            'Replace',
                                                                                                            'None']),
                   help='The default principals modification kind')
        c.argument('table_level_sharing_properties', options_list=['--table-level-sharing-properties', '--tlsp'],
                   action=AddTableLevelSharingProperties, nargs='+', help='Table level sharing specifications')

    with self.argument_context('synapse kusto attached-database-configuration update') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('attached_database_configuration_name', options_list=['--attached-database-configuration-name',
                                                                         '--adcn'], type=str, help='The name of the '
                   'attached database configuration.', id_part='child_name_2')
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('database_name', type=str, help='The name of the database which you would like to attach, use * if '
                   'you want to follow all current and future databases.')
        c.argument('kusto_pool_resource_id', type=str, help='The resource id of the kusto pool where the databases you '
                   'would like to attach reside.')
        c.argument('default_principals_modification_kind', options_list=['--default-principals-modification-kind',
                                                                         '--dpmk'], arg_type=get_enum_type(['Union',
                                                                                                            'Replace',
                                                                                                            'None']),
                   help='The default principals modification kind')
        c.argument('table_level_sharing_properties', options_list=['--table-level-sharing-properties', '--tlsp'],
                   action=AddTableLevelSharingProperties, nargs='+', help='Table level sharing specifications')
        c.ignore('parameters')

    with self.argument_context('synapse kusto attached-database-configuration delete') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('attached_database_configuration_name', options_list=['--attached-database-configuration-name',
                                                                         '--adcn'], type=str, help='The name of the '
                   'attached database configuration.', id_part='child_name_2')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto attached-database-configuration wait') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('attached_database_configuration_name', options_list=['--attached-database-configuration-name',
                                                                         '--adcn'], type=str, help='The name of the '
                   'attached database configuration.', id_part='child_name_2')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto database list') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')

    with self.argument_context('synapse kusto database show') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')

    with self.argument_context('synapse kusto database create') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.')
        c.argument('read_write_database', action=AddReadWriteDatabase, nargs='+', help='Class representing a read '
                   'write database.', arg_group='Parameters')

    with self.argument_context('synapse kusto database update') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('read_write_database', action=AddReadWriteDatabase, nargs='+', help='Class representing a read '
                   'write database.', arg_group='Parameters')

    with self.argument_context('synapse kusto database delete') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')

    with self.argument_context('synapse kusto database wait') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')

    with self.argument_context('synapse kusto data-connection list') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.')

    with self.argument_context('synapse kusto data-connection show') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('data_connection_name', type=str, help='The name of the data connection.', id_part='child_name_3')

    with self.argument_context('synapse kusto data-connection event-grid create') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.')
        c.argument('data_connection_name', type=str, help='The name of the data connection.')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('storage_account_resource_id', options_list=['--storage-account-resource-id', '--sari'], type=str,
                   help='The resource ID of the storage account where the data resides.')
        c.argument('event_hub_resource_id', type=str, help='The resource ID where the event grid is configured to send '
                   'events.')
        c.argument('consumer_group', type=str, help='The event hub consumer group.')
        c.argument('table_name', type=str, help='The table where the data should be ingested. Optionally the table '
                   'information can be added to each message.')
        c.argument('mapping_rule_name', type=str, help='The mapping rule to be used to ingest the data. Optionally the '
                   'mapping information can be added to each message.')
        c.argument('data_format', arg_type=get_enum_type(['MULTIJSON', 'JSON', 'CSV', 'TSV', 'SCSV', 'SOHSV', 'PSV',
                                                          'TXT', 'RAW', 'SINGLEJSON', 'AVRO', 'TSVE', 'PARQUET', 'ORC',
                                                          'APACHEAVRO', 'W3CLOGFILE']), help='The data format of the '
                   'message. Optionally the data format can be added to each message.')
        c.argument('ignore_first_record', arg_type=get_three_state_flag(), help='A Boolean value that, if set to true, '
                   'indicates that ingestion should ignore the first record of every file')
        c.argument('blob_storage_event_type', options_list=['--blob-storage-event-type', '--bset'],
                   arg_type=get_enum_type(['Microsoft.Storage.BlobCreated', 'Microsoft.Storage.BlobRenamed']),
                   help='The name of blob storage event type to process.')

    with self.argument_context('synapse kusto data-connection event-hub create') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.')
        c.argument('data_connection_name', type=str, help='The name of the data connection.')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('event_hub_resource_id', type=str, help='The resource ID of the event hub to be used to create a '
                   'data connection.')
        c.argument('consumer_group', type=str, help='The event hub consumer group.')
        c.argument('table_name', type=str, help='The table where the data should be ingested. Optionally the table '
                   'information can be added to each message.')
        c.argument('mapping_rule_name', type=str, help='The mapping rule to be used to ingest the data. Optionally the '
                   'mapping information can be added to each message.')
        c.argument('data_format', arg_type=get_enum_type(['MULTIJSON', 'JSON', 'CSV', 'TSV', 'SCSV', 'SOHSV', 'PSV',
                                                          'TXT', 'RAW', 'SINGLEJSON', 'AVRO', 'TSVE', 'PARQUET', 'ORC',
                                                          'APACHEAVRO', 'W3CLOGFILE']), help='The data format of the '
                   'message. Optionally the data format can be added to each message.')
        c.argument('event_system_properties', options_list=['--event-system-properties', '--esysp'], nargs='+',
                   help='System properties of the event hub')
        c.argument('compression', arg_type=get_enum_type(['None', 'GZip']), help='The event hub messages compression '
                   'type')
        c.argument('managed_identity_resource_id', options_list=['--managed-identity-resource-id', '--miri'], type=str,
                   help='The resource ID of a managed identity (system or user assigned) to be used to authenticate '
                   'with event hub.')

    with self.argument_context('synapse kusto data-connection iot-hub create') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.')
        c.argument('data_connection_name', type=str, help='The name of the data connection.')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('iot_hub_resource_id', type=str, help='The resource ID of the Iot hub to be used to create a data '
                   'connection.')
        c.argument('consumer_group', type=str, help='The iot hub consumer group.')
        c.argument('table_name', type=str, help='The table where the data should be ingested. Optionally the table '
                   'information can be added to each message.')
        c.argument('mapping_rule_name', type=str, help='The mapping rule to be used to ingest the data. Optionally the '
                   'mapping information can be added to each message.')
        c.argument('data_format', arg_type=get_enum_type(['MULTIJSON', 'JSON', 'CSV', 'TSV', 'SCSV', 'SOHSV', 'PSV',
                                                          'TXT', 'RAW', 'SINGLEJSON', 'AVRO', 'TSVE', 'PARQUET', 'ORC',
                                                          'APACHEAVRO', 'W3CLOGFILE']), help='The data format of the '
                   'message. Optionally the data format can be added to each message.')
        c.argument('event_system_properties', options_list=['--event-system-properties', '--esysp'], nargs='+',
                   help='System properties of the iot hub')
        c.argument('shared_access_policy_name', options_list=['--shared-access-policy-name', '--sapn'], type=str,
                   help='The name of the share access policy')

    with self.argument_context('synapse kusto data-connection event-grid update') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('data_connection_name', type=str, help='The name of the data connection.', id_part='child_name_3')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('storage_account_resource_id', options_list=['--storage-account-resource-id', '--sari'], type=str,
                   help='The resource ID of the storage account where the data resides.')
        c.argument('event_hub_resource_id', type=str, help='The resource ID where the event grid is configured to send '
                   'events.')
        c.argument('consumer_group', type=str, help='The event hub consumer group.')
        c.argument('table_name', type=str, help='The table where the data should be ingested. Optionally the table '
                   'information can be added to each message.')
        c.argument('mapping_rule_name', type=str, help='The mapping rule to be used to ingest the data. Optionally the '
                   'mapping information can be added to each message.')
        c.argument('data_format', arg_type=get_enum_type(['MULTIJSON', 'JSON', 'CSV', 'TSV', 'SCSV', 'SOHSV', 'PSV',
                                                          'TXT', 'RAW', 'SINGLEJSON', 'AVRO', 'TSVE', 'PARQUET', 'ORC',
                                                          'APACHEAVRO', 'W3CLOGFILE']), help='The data format of the '
                   'message. Optionally the data format can be added to each message.')
        c.argument('ignore_first_record', arg_type=get_three_state_flag(), help='A Boolean value that, if set to true, '
                   'indicates that ingestion should ignore the first record of every file')
        c.argument('blob_storage_event_type', options_list=['--blob-storage-event-type', '--bset'],
                   arg_type=get_enum_type(['Microsoft.Storage.BlobCreated', 'Microsoft.Storage.BlobRenamed']),
                   help='The name of blob storage event type to process.')

    with self.argument_context('synapse kusto data-connection event-hub update') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('data_connection_name', type=str, help='The name of the data connection.', id_part='child_name_3')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('event_hub_resource_id', type=str, help='The resource ID of the event hub to be used to create a '
                   'data connection.')
        c.argument('consumer_group', type=str, help='The event hub consumer group.')
        c.argument('table_name', type=str, help='The table where the data should be ingested. Optionally the table '
                   'information can be added to each message.')
        c.argument('mapping_rule_name', type=str, help='The mapping rule to be used to ingest the data. Optionally the '
                   'mapping information can be added to each message.')
        c.argument('data_format', arg_type=get_enum_type(['MULTIJSON', 'JSON', 'CSV', 'TSV', 'SCSV', 'SOHSV', 'PSV',
                                                          'TXT', 'RAW', 'SINGLEJSON', 'AVRO', 'TSVE', 'PARQUET', 'ORC',
                                                          'APACHEAVRO', 'W3CLOGFILE']), help='The data format of the '
                   'message. Optionally the data format can be added to each message.')
        c.argument('event_system_properties', options_list=['--event-system-properties', '--esysp'], nargs='+',
                   help='System properties of the event hub')
        c.argument('compression', arg_type=get_enum_type(['None', 'GZip']), help='The event hub messages compression '
                   'type')
        c.argument('managed_identity_resource_id', options_list=['--managed-identity-resource-id', '--miri'], type=str,
                   help='The resource ID of a managed identity (system or user assigned) to be used to authenticate '
                   'with event hub.')

    with self.argument_context('synapse kusto data-connection iot-hub update') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('data_connection_name', type=str, help='The name of the data connection.', id_part='child_name_3')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('iot_hub_resource_id', type=str, help='The resource ID of the Iot hub to be used to create a data '
                   'connection.')
        c.argument('consumer_group', type=str, help='The iot hub consumer group.')
        c.argument('table_name', type=str, help='The table where the data should be ingested. Optionally the table '
                   'information can be added to each message.')
        c.argument('mapping_rule_name', type=str, help='The mapping rule to be used to ingest the data. Optionally the '
                   'mapping information can be added to each message.')
        c.argument('data_format', arg_type=get_enum_type(['MULTIJSON', 'JSON', 'CSV', 'TSV', 'SCSV', 'SOHSV', 'PSV',
                                                          'TXT', 'RAW', 'SINGLEJSON', 'AVRO', 'TSVE', 'PARQUET', 'ORC',
                                                          'APACHEAVRO', 'W3CLOGFILE']), help='The data format of the '
                   'message. Optionally the data format can be added to each message.')
        c.argument('event_system_properties', options_list=['--event-system-properties', '--esysp'], nargs='+',
                   help='System properties of the iot hub')
        c.argument('shared_access_policy_name', options_list=['--shared-access-policy-name', '--sapn'], type=str,
                   help='The name of the share access policy')

    with self.argument_context('synapse kusto data-connection delete') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('data_connection_name', type=str, help='The name of the data connection.', id_part='child_name_3')

    with self.argument_context('synapse kusto data-connection wait') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('data_connection_name', type=str, help='The name of the data connection.', id_part='child_name_3')

    with self.argument_context('synapse kusto pool-principal-assignment list') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool-principal-assignment show') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.', id_part='child_name_2')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool-principal-assignment create') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.')
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('principal_id', type=str, help='The principal ID assigned to the cluster principal. It can be a '
                   'user email, application ID, or security group name.')
        c.argument('role', arg_type=get_enum_type(['AllDatabasesAdmin', 'AllDatabasesViewer']), help='Cluster '
                   'principal role.')
        c.argument('tenant_id', type=str, help='The tenant id of the principal')
        c.argument('principal_type', arg_type=get_enum_type(['App', 'Group', 'User']), help='Principal type.')

    with self.argument_context('synapse kusto pool-principal-assignment update') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.', id_part='child_name_2')
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('principal_id', type=str, help='The principal ID assigned to the cluster principal. It can be a '
                   'user email, application ID, or security group name.')
        c.argument('role', arg_type=get_enum_type(['AllDatabasesAdmin', 'AllDatabasesViewer']), help='Cluster '
                   'principal role.')
        c.argument('tenant_id', type=str, help='The tenant id of the principal')
        c.argument('principal_type', arg_type=get_enum_type(['App', 'Group', 'User']), help='Principal type.')
        c.ignore('parameters')

    with self.argument_context('synapse kusto pool-principal-assignment delete') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.', id_part='child_name_2')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto pool-principal-assignment wait') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.', id_part='child_name_2')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto database-principal-assignment list') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto database-principal-assignment show') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.', id_part='child_name_3')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto database-principal-assignment create') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.')
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('principal_id', type=str, help='The principal ID assigned to the database principal. It can be a '
                   'user email, application ID, or security group name.')
        c.argument('role', arg_type=get_enum_type(['Admin', 'Ingestor', 'Monitor', 'User', 'UnrestrictedViewer',
                                                   'Viewer']), help='Database principal role.')
        c.argument('tenant_id', type=str, help='The tenant id of the principal')
        c.argument('principal_type', arg_type=get_enum_type(['App', 'Group', 'User']), help='Principal type.')

    with self.argument_context('synapse kusto database-principal-assignment update') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.', id_part='child_name_3')
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('principal_id', type=str, help='The principal ID assigned to the database principal. It can be a '
                   'user email, application ID, or security group name.')
        c.argument('role', arg_type=get_enum_type(['Admin', 'Ingestor', 'Monitor', 'User', 'UnrestrictedViewer',
                                                   'Viewer']), help='Database principal role.')
        c.argument('tenant_id', type=str, help='The tenant id of the principal')
        c.argument('principal_type', arg_type=get_enum_type(['App', 'Group', 'User']), help='Principal type.')
        c.ignore('parameters')

    with self.argument_context('synapse kusto database-principal-assignment delete') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.', id_part='child_name_3')
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('synapse kusto database-principal-assignment wait') as c:
        c.argument('workspace_name', type=str, help='The name of the workspace', id_part='name')
        c.argument('kusto_pool_name', type=str, help='The name of the Kusto pool.', id_part='child_name_1')
        c.argument('database_name', type=str, help='The name of the database in the Kusto pool.',
                   id_part='child_name_2')
        c.argument('principal_assignment_name', options_list=['--principal-assignment-name', '--psn'], type=str,
                   help='The name of the Kusto principalAssignment.', id_part='child_name_3')
        c.argument('resource_group_name', resource_group_name_type)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
# pylint: skip-file
# flake8: noqa
import warnings

from .v7_2.models import *



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from collections.abc import Iterable
from azure.cli.command_modules.keyvault._validators import KeyEncryptionDataType


def multi_transformers(*transformers):
    def _multi_transformers(output, **command_args):
        for t in transformers:
            output = t(output, **command_args)
        return output
    return _multi_transformers


def keep_max_results(output, **command_args):
    maxresults = command_args.get('maxresults', None)
    if maxresults:
        return list(output)[:maxresults]
    return output


def filter_out_managed_resources(output, **command_args):  # pylint: disable=unused-argument
    is_kv_transform = command_args.get('kv_transform')
    if is_kv_transform:
        if command_args.get('include_managed'):
            return output
        return [_ for _ in output if not getattr(_, 'managed')] if output else output
    return output


def _extract_subresource_name_from_single_output(output, id_parameter):
    if not getattr(output, id_parameter):
        resource_name = None
    else:
        items = getattr(output, id_parameter).split('/')
        resource_name = items[4] if len(items) > 4 else None

    setattr(output, 'name', resource_name)
    return output


def extract_subresource_name(id_parameter='id'):
    def _extract_subresource_name(output, **command_args):  # pylint: disable=unused-argument
        if isinstance(output, Iterable):
            return [_extract_subresource_name_from_single_output(item, id_parameter) for item in output]
        return _extract_subresource_name_from_single_output(output, id_parameter)
    return _extract_subresource_name


def transform_key_decryption_output(output, **command_args):
    data_type = command_args.get('data_type')
    if data_type == KeyEncryptionDataType.BASE64:
        return output
    raw_result = getattr(output, 'result')
    if not raw_result or not isinstance(raw_result, bytes):
        return output
    setattr(output, 'result', raw_result.decode('utf-8'))
    return output


# pylint: disable=unused-argument, protected-access
def transform_key_output(result, **command_args):
    from azure.keyvault.keys import KeyVaultKey, JsonWebKey
    import base64

    if not isinstance(result, KeyVaultKey):
        return result

    if result.key and isinstance(result.key, JsonWebKey):
        for attr in result.key._FIELDS:
            value = getattr(result.key, attr)
            if value and isinstance(value, bytes):
                setattr(result.key, attr, base64.b64encode(value))

    output = {
        'attributes': result.properties._attributes,
        'key': result.key,
        'managed': result.properties.managed,
        'tags': result.properties.tags,
        'releasePolicy': result.properties.release_policy
    }
    return output



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: skip-file
# flake8: noqa

from .security_domain_certificate_item_py3 import SecurityDomainCertificateItem
from .security_domain_json_web_key_py3 import SecurityDomainJsonWebKey
from .security_domain_object_py3 import SecurityDomainObject
from .security_domain_operation_status_py3 import SecurityDomainOperationStatus
from .transfer_key_py3 import TransferKey
from .certificate_set_py3 import CertificateSet
from .error_py3 import Error
from .key_vault_error_py3 import KeyVaultError, KeyVaultErrorException


from .key_vault_client_enums import (
    OperationStatus
)

__all__ = [
    'SecurityDomainCertificateItem',
    'SecurityDomainJsonWebKey',
    'SecurityDomainObject',
    'SecurityDomainOperationStatus',
    'TransferKey',
    'CertificateSet',
    'Error',
    'KeyVaultError',
    'KeyVaultErrorException',
    'OperationStatus'
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.core.commands import CliCommandType


# pylint: disable=line-too-long, too-many-statements, too-many-locals
def load_command_table(self, _):
    from ._client_factory import cf_synapse_client_workspace_factory
    from ._client_factory import cf_synapse_client_operations_factory
    from ._client_factory import cf_synapse_client_bigdatapool_factory
    from ._client_factory import cf_synapse_client_sqlpool_factory
    from ._client_factory import cf_synapse_client_ipfirewallrules_factory
    from ._client_factory import cf_synapse_client_cmk_factory
    from ._client_factory import cf_synapse_client_sqlpool_sensitivity_labels_factory
    from ._client_factory import cf_synapse_client_restorable_dropped_sqlpools_factory
    from ._client_factory import cf_synapse_client_sqlpool_transparent_data_encryptions_factory
    from ._client_factory import cf_synapse_client_sqlpool_security_alert_policies_factory
    from ._client_factory import cf_synapse_client_sqlpool_blob_auditing_policies_factory
    from ._client_factory import cf_synapse_client_managed_identity_sqlcontrol_factory
    from ._client_factory import cf_synapse_client_workspace_aad_admins_factory
    from ._client_factory import cf_synapse_client_sqlserver_blob_auditing_policies_factory
    from ._client_factory import cf_synapse_client_integrationruntimes_factory
    from ._client_factory import cf_synapse_client_integrationruntimeauthkeys_factory
    from ._client_factory import cf_synapse_client_integrationruntimemonitoringdata_factory
    from ._client_factory import cf_synapse_client_integrationruntimenodeipaddress_factory
    from ._client_factory import cf_synapse_client_integrationruntimenodes_factory
    from ._client_factory import cf_synapse_client_integrationruntimecredentials_factory
    from ._client_factory import cf_synapse_client_integrationruntimeconnectioninfos_factory
    from ._client_factory import cf_synapse_client_integrationruntimestatus_factory
    from ._client_factory import cf_kusto_pool

    def get_custom_sdk(custom_module, client_factory):
        return CliCommandType(
            operations_tmpl='azure.cli.command_modules.synapse.manual.operations.{}#'.format(custom_module) + '{}',
            client_factory=client_factory,
        )

    synapse_workspace_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#WorkspacesOperations.{}',
        client_factory=cf_synapse_client_workspace_factory)

    synapse_operations_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#Operations.{}',
        client_factory=cf_synapse_client_operations_factory)

    synapse_bigdatapool_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#BigDataPoolsOperations.{}',
        client_factory=cf_synapse_client_bigdatapool_factory)

    synapse_workspace_aad_admin_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#WorkspaceAadAdminsOperations.{}'
    )

    synapse_sqlpool_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#SqlPoolsOperations.{}',
        client_factory=cf_synapse_client_sqlpool_factory)

    # Classification operation
    synapse_sqlpool_sensitivity_labels_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#SqlPoolSensitivityLabelsOperations.{}',
        client_factory=cf_synapse_client_sqlpool_sensitivity_labels_factory)

    # List deleted
    synapse_restorable_dropped_sqlpools_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#RestorableDroppedSqlPoolsOperations.{}',
        client_factory=cf_synapse_client_restorable_dropped_sqlpools_factory)

    # Tde operation
    synapse_sqlpool_transparent_data_encryptions_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#SqlPoolTransparentDataEncryptionsOperations.{}',
        client_factory=cf_synapse_client_sqlpool_transparent_data_encryptions_factory)

    # Threat policy operation
    synapse_sqlpool_security_alert_policies_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#SqlPoolSecurityAlertPoliciesOperations.{}',
        client_factory=cf_synapse_client_sqlpool_security_alert_policies_factory)

    # Audit policy operation
    synapse_sqlpool_blob_auditing_policies_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#SqlPoolBlobAuditingPoliciesOperations.{}',
        client_factory=cf_synapse_client_sqlpool_blob_auditing_policies_factory)

    # Workspace managed sql server audit policy operation
    synapse_workspace_managed_sqlserver_blob_auditing_policies_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#WorkspaceManagedSqlServerBlobAuditingPoliciesOperations.{}',
        client_factory=cf_synapse_client_sqlserver_blob_auditing_policies_factory)

    synapse_firewallrules_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IpFirewallRulesOperations.{}',
        client_factory=cf_synapse_client_ipfirewallrules_factory)

    synapse_cmk_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#KeysOperations.{}',
        client_factory=cf_synapse_client_cmk_factory)

    synapse_managedidentitysqlcontrol_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#WorkspaceManagedIdentitySqlControlSettingsOperations.{}',
        client_factory=cf_synapse_client_managed_identity_sqlcontrol_factory)

    synapse_integrationruntimes_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IntegrationRuntimesOperations.{}',
        client_factory=cf_synapse_client_integrationruntimes_factory)

    synapse_integrationruntimeauthkeys_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IntegrationRuntimeAuthKeysOperations.{}',
        client_factory=cf_synapse_client_integrationruntimeauthkeys_factory)

    synapse_integrationruntimemonitoringdata_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IntegrationRuntimeMonitoringDataOperations.{}',
        client_factory=cf_synapse_client_integrationruntimemonitoringdata_factory)

    synapse_integrationruntimenodes_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IntegrationRuntimeNodesOperations.{}',
        client_factory=cf_synapse_client_integrationruntimenodes_factory)

    synapse_integrationruntimenodeipaddress_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IntegrationRuntimeNodeIpAddressOperations.{}',
        client_factory=cf_synapse_client_integrationruntimenodeipaddress_factory)

    synapse_integrationruntimecredentials_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IntegrationRuntimeCredentialsOperations.{}',
        client_factory=cf_synapse_client_integrationruntimecredentials_factory)

    synapse_integrationruntimeconnectioninfos_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IntegrationRuntimeConnectionInfosOperations.{}',
        client_factory=cf_synapse_client_integrationruntimeconnectioninfos_factory)

    synapse_integrationruntimestatus_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations#IntegrationRuntimeStatusOperations.{}',
        client_factory=cf_synapse_client_integrationruntimestatus_factory)

    synapse_spark_session_sdk = CliCommandType(
        operations_tmpl='azure.synapse.spark.operations#SparkSessionOperations.{}',
        client_factory=None)

    synapse_spark_batch_sdk = CliCommandType(
        operations_tmpl='azure.synapse.spark.operations#SparkBatchOperations.{}',
        client_factory=None)

    synapse_role_assignment_sdk = CliCommandType(
        operations_tmpl='azure.synapse.accesscontrol.operations#RoleAssignmentsOperations.{}',
        client_factory=None)

    synapse_role_definitions_sdk = CliCommandType(
        operations_tmpl='azure.synapse.accesscontrol.operations#RoleDefinitionsOperations.{}',
        client_factory=None)

    synapse_linked_service_sdk = CliCommandType(
        operation_tmpl='azure.synapse.artifacts.operations#LinkedServiceOperations.{}',
        client_factory=None)

    synapse_dataset_sdk = CliCommandType(
        operation_tmpl='azure.synapse.artifacts.operations#DatasetOperations.{}',
        client_factory=None)

    synapse_pipeline_sdk = CliCommandType(
        operation_tmpl='azure.synapse.artifacts.operations#PipelineOperations.{}',
        client_factory=None)

    synapse_pipeline_run_sdk = CliCommandType(
        operation_tmpl='azure.synapse.artifacts.operations#PipelineRunOperations.{}',
        client_factory=None)

    synapse_trigger_sdk = CliCommandType(
        operations_tmpl='azure.synapse.artifacts.operations#TriggerOperations.{}',
        client_factory=None)

    synapse_data_flow_sdk = CliCommandType(
        operation_tmpl='azure.synapse.artifacts.operations#DataFlowOperations.{}',
        client_factory=None)

    synapse_trigger_run_sdk = CliCommandType(
        operation_tmpl='azure.synapse.artifacts.operations#TriggerRunOperations.{}',
        client_factory=None)

    synapse_notebook_sdk = CliCommandType(
        operation_tmpl='azure.synapse.artifacts.operations#NotebookOperations.{}',
        client_factory=None)

    synapse_library_sdk = CliCommandType(
        operation_tmpl='azure.synapse.artifacts.operations#LibraryOperations.{}',
        client_factory=None)

    synapse_managed_private_endpoints_sdk = CliCommandType(
        operations_tmpl='azure.synapse.managedprivateendpoints.operations#ManagedPrivateEndpoints.{}',
        client_factory=None)

    synapse_spark_job_definition_sdk = CliCommandType(
        operations_tmpl='azure.synapse.artifacts.operations#SparkJobDefinitionOperations.{}',
        client_factory=None)

    synapse_kusto_pool_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.synapse.operations._kusto_pools_operations#KustoPoolsOperations.{}',
        client_factory=cf_kusto_pool,
    )
    # Management Plane Commands --Workspace
    with self.command_group('synapse workspace', command_type=synapse_workspace_sdk,
                            custom_command_type=get_custom_sdk('workspace', cf_synapse_client_workspace_factory),
                            client_factory=cf_synapse_client_workspace_factory) as g:
        g.show_command('show', 'get')
        g.custom_command('list', 'list_workspaces')
        g.custom_command('create', 'create_workspace', supports_no_wait=True)
        g.custom_command('update', 'update_workspace', supports_no_wait=True)
        g.custom_command('check-name', 'custom_check_name_availability',
                         command_type=synapse_operations_sdk,
                         client_factory=cf_synapse_client_operations_factory)
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.custom_command('activate', 'activate_workspace', command_type=synapse_cmk_sdk, client_factory=cf_synapse_client_cmk_factory, supports_no_wait=True)
        g.wait_command('wait')

    # Management Plane Commands --SparkPool
    with self.command_group('synapse spark pool', command_type=synapse_bigdatapool_sdk,
                            custom_command_type=get_custom_sdk('sparkpool', cf_synapse_client_bigdatapool_factory),
                            client_factory=cf_synapse_client_bigdatapool_factory) as g:
        g.custom_show_command('show', 'get_spark_pool')
        g.command('list', 'list_by_workspace')
        g.custom_command('create', 'create_spark_pool', supports_no_wait=True)
        g.custom_command('update', 'update_spark_pool', supports_no_wait=True)
        g.custom_command('delete', 'delete_spark_pool', confirmation=True, supports_no_wait=True)
        g.wait_command('wait')

    # Management Plane Commands --SqlPool
    with self.command_group('synapse sql pool', command_type=synapse_sqlpool_sdk,
                            custom_command_type=get_custom_sdk('sqlpool', cf_synapse_client_sqlpool_factory),
                            client_factory=cf_synapse_client_sqlpool_factory) as g:
        g.show_command('show', 'get')
        g.command('list', 'list_by_workspace')
        g.custom_command('create', 'create_sql_pool', supports_no_wait=True)
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.custom_command('update', 'update_sql_pool')
        g.command('pause', 'begin_pause')
        g.command('resume', 'begin_resume')
        g.custom_command('restore', 'restore_sql_pool')
        g.custom_command('show-connection-string', 'sql_pool_show_connection_string')
        g.wait_command('wait')

    # Management Plane Commands --SqlPool list-deleted
    with self.command_group('synapse sql pool', command_type=synapse_restorable_dropped_sqlpools_sdk,
                            client_factory=cf_synapse_client_restorable_dropped_sqlpools_factory) as g:
        g.command('list-deleted', 'list_by_workspace')

    #  Management Plane Commands --SqlPool Classification
    with self.command_group('synapse sql pool classification', command_type=synapse_sqlpool_sensitivity_labels_sdk,
                            custom_command_type=get_custom_sdk('sqlpoolsensitivitylabel',
                                                               cf_synapse_client_sqlpool_sensitivity_labels_factory),
                            client_factory=cf_synapse_client_sqlpool_sensitivity_labels_factory) as g:
        g.custom_show_command('show', 'sqlpool_sensitivity_label_show')
        g.command('list', 'list_current')
        g.custom_command('create', 'sqlpool_sensitivity_label_create')
        g.command('delete', 'delete')
        g.custom_command('update', 'sqlpool_sensitivity_label_update')

    with self.command_group('synapse sql pool classification recommendation',
                            command_type=synapse_sqlpool_sensitivity_labels_sdk,
                            custom_command_type=get_custom_sdk('sqlpoolsensitivitylabel',
                                                               cf_synapse_client_sqlpool_sensitivity_labels_factory),
                            client_factory=cf_synapse_client_sqlpool_sensitivity_labels_factory) as g:
        g.command('list', 'list_recommended')
        g.command('enable', 'enable_recommendation')
        g.command('disable', 'disable_recommendation')

    #  Management Plane Commands --SqlPool Tde
    with self.command_group('synapse sql pool tde', command_type=synapse_sqlpool_transparent_data_encryptions_sdk,
                            custom_command_type=get_custom_sdk('sqlpooltde',
                                                               cf_synapse_client_sqlpool_transparent_data_encryptions_factory),
                            client_factory=cf_synapse_client_sqlpool_transparent_data_encryptions_factory) as g:
        g.custom_command('set', 'create_or_update')
        g.show_command('show', 'get')

    #  Management Plane Commands --SqlPool Threat-policy
    with self.command_group('synapse sql pool threat-policy', command_type=synapse_sqlpool_security_alert_policies_sdk,
                            custom_command_type=get_custom_sdk('sqlpoolsecurityalertpolicy',
                                                               cf_synapse_client_sqlpool_security_alert_policies_factory),
                            client_factory=cf_synapse_client_sqlpool_security_alert_policies_factory) as g:
        g.show_command('show', 'get')
        g.generic_update_command('update', custom_func_name='sqlpool_security_alert_policy_update')

    #  Management Plane Commands --SqlPool Audit-policy
    with self.command_group('synapse sql pool audit-policy', command_type=synapse_sqlpool_blob_auditing_policies_sdk,
                            custom_command_type=get_custom_sdk('sqlpoolblobauditingpolicy',
                                                               cf_synapse_client_sqlpool_blob_auditing_policies_factory),
                            client_factory=cf_synapse_client_sqlpool_blob_auditing_policies_factory) as g:
        g.custom_show_command('show', 'sqlpool_audit_policy_show')
        g.generic_update_command('update', custom_func_name='sqlpool_blob_auditing_policy_update')

    # Management Plane Commands --Sql Ad-Admin
    with self.command_group('synapse sql ad-admin', command_type=synapse_workspace_aad_admin_sdk,
                            custom_command_type=get_custom_sdk('workspacesqlaadadmin',
                                                               cf_synapse_client_workspace_aad_admins_factory),
                            client_factory=cf_synapse_client_workspace_aad_admins_factory) as g:
        g.show_command('show', 'get')
        g.custom_command('create', 'create_workspace_sql_aad_admin', supports_no_wait=True)
        g.generic_update_command('update', setter_name='begin_create_or_update', custom_func_name='update_workspace_sql_aad_admin',
                                 setter_arg_name='aad_admin_info', supports_no_wait=True)
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.wait_command('wait')

    # Management Plane Commands --Sql audit-policy
    with self.command_group('synapse sql audit-policy',
                            command_type=synapse_workspace_managed_sqlserver_blob_auditing_policies_sdk,
                            custom_command_type=get_custom_sdk('sqlpoolblobauditingpolicy',
                                                               cf_synapse_client_sqlserver_blob_auditing_policies_factory),
                            client_factory=cf_synapse_client_sqlserver_blob_auditing_policies_factory) as g:
        g.custom_show_command('show', 'workspace_audit_policy_show')
        g.generic_update_command('update', setter_name='begin_create_or_update', custom_func_name='sqlserver_blob_auditing_policy_update',
                                 supports_no_wait=True)
        g.wait_command('wait')

    # Management Plane Commands --FirewallRule
    with self.command_group('synapse workspace firewall-rule', command_type=synapse_firewallrules_sdk,
                            custom_command_type=get_custom_sdk('workspace', cf_synapse_client_ipfirewallrules_factory),
                            client_factory=cf_synapse_client_ipfirewallrules_factory) as g:
        g.command('list', 'list_by_workspace')
        g.show_command('show', 'get')
        g.custom_command('create', 'create_firewall_rule', supports_no_wait=True)
        g.custom_command('update', 'update_firewall_rule', supports_no_wait=True)
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.wait_command('wait')

    # Management Plane Commands --IntegrationRuntime
    with self.command_group('synapse integration-runtime', command_type=synapse_integrationruntimes_sdk,
                            custom_command_type=get_custom_sdk('integrationruntime', cf_synapse_client_integrationruntimes_factory),
                            client_factory=cf_synapse_client_integrationruntimes_factory) as g:
        g.command('list', 'list_by_workspace')
        g.show_command('show', 'get')
        g.custom_command('create', 'create', deprecate_info=g.deprecate(redirect='managed create， self-hosted create'), supports_no_wait=True)
        g.custom_command('managed create', 'Managed_Create', supports_no_wait=True)
        g.custom_command('self-hosted create', 'Selfhosted_Create', supports_no_wait=True)
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.custom_command('update', 'update')
        g.command('start', 'begin_start', supports_no_wait=True)
        g.command('stop', 'begin_stop', confirmation=True, supports_no_wait=True)
        g.command('upgrade', 'upgrade')
        g.command('list-auth-key', 'list', command_type=synapse_integrationruntimeauthkeys_sdk,
                  client_factory=cf_synapse_client_integrationruntimeauthkeys_factory)
        g.custom_command('regenerate-auth-key', 'regenerate', command_type=synapse_integrationruntimeauthkeys_sdk,
                         client_factory=cf_synapse_client_integrationruntimeauthkeys_factory)
        g.command('get-monitoring-data', 'list', command_type=synapse_integrationruntimemonitoringdata_sdk,
                  client_factory=cf_synapse_client_integrationruntimemonitoringdata_factory)
        g.command('sync-credentials', 'sync', command_type=synapse_integrationruntimecredentials_sdk,
                  client_factory=cf_synapse_client_integrationruntimecredentials_factory)
        g.command('get-connection-info', 'get', command_type=synapse_integrationruntimeconnectioninfos_sdk,
                  client_factory=cf_synapse_client_integrationruntimeconnectioninfos_factory)
        g.command('get-status', 'get', command_type=synapse_integrationruntimestatus_sdk,
                  client_factory=cf_synapse_client_integrationruntimestatus_factory)
        g.wait_command('wait')

    # Management Plane Commands --Keys
    with self.command_group('synapse workspace key', command_type=synapse_cmk_sdk,
                            custom_command_type=get_custom_sdk('workspace', cf_synapse_client_cmk_factory),
                            client_factory=cf_synapse_client_cmk_factory) as g:
        g.command('list', 'list_by_workspace')
        g.show_command('show', 'get')
        g.custom_command('create', 'create_workspace_key', supports_no_wait=True)
        g.command('delete', 'delete', confirmation=True, supports_no_wait=True)
        g.wait_command('wait')

    # Management Plane Commands --Managed-Identity
    with self.command_group('synapse workspace managed-identity', command_type=synapse_managedidentitysqlcontrol_sdk,
                            custom_command_type=get_custom_sdk('workspace', cf_synapse_client_managed_identity_sqlcontrol_factory),
                            client_factory=cf_synapse_client_managed_identity_sqlcontrol_factory) as g:
        g.show_command('show-sql-access', 'get')
        g.custom_command('grant-sql-access', 'grant_sql_access_to_managed_identity', supports_no_wait=True)
        g.custom_command('revoke-sql-access', 'revoke_sql_access_to_managed_identity', supports_no_wait=True)
        g.wait_command('wait')

    with self.command_group('synapse integration-runtime-node', command_type=synapse_integrationruntimenodes_sdk,
                            custom_command_type=get_custom_sdk('integrationruntimenode',
                                                               cf_synapse_client_integrationruntimenodes_factory),
                            client_factory=cf_synapse_client_integrationruntimenodes_factory) as g:
        g.show_command('show', 'get')
        g.custom_command('update', 'update')
        g.command('delete', 'delete', confirmation=True)
        g.command('get-ip-address', 'get', command_type=synapse_integrationruntimenodeipaddress_sdk,
                  client_factory=cf_synapse_client_integrationruntimenodeipaddress_factory)

    # Data Plane Commands --Spark batch opertions
    with self.command_group('synapse spark job', command_type=synapse_spark_batch_sdk,
                            custom_command_type=get_custom_sdk('spark', None)) as g:
        g.custom_command('submit', 'create_spark_batch_job')
        g.custom_command('list', 'list_spark_batch_jobs')
        g.custom_show_command('show', 'get_spark_batch_job')
        g.custom_command('cancel', 'cancel_spark_batch_job', confirmation=True)

    # Data Plane Commands --Spark session operations
    with self.command_group('synapse spark session', synapse_spark_session_sdk,
                            custom_command_type=get_custom_sdk('spark', None)) as g:
        g.custom_command('create', 'create_spark_session_job')
        g.custom_command('list', 'list_spark_session_jobs')
        g.custom_show_command('show', 'get_spark_session_job')
        g.custom_command('cancel', 'cancel_spark_session_job', confirmation=True)
        g.custom_command('reset-timeout', 'reset_timeout')

    # Data Plane Commands --Spark session statements operations
    with self.command_group('synapse spark statement', synapse_spark_session_sdk,
                            custom_command_type=get_custom_sdk('spark', None)) as g:
        g.custom_command('invoke', 'create_spark_session_statement')
        g.custom_command('list', 'list_spark_session_statements')
        g.custom_show_command('show', 'get_spark_session_statement')
        g.custom_command('cancel', 'cancel_spark_session_statement', confirmation=True)

    # Data Plane Commands --Access control operations
    with self.command_group('synapse role assignment', synapse_role_assignment_sdk,
                            custom_command_type=get_custom_sdk('accesscontrol', None)) as g:
        g.custom_command('create', 'create_role_assignment')
        g.custom_command('list', 'list_role_assignments')
        g.custom_show_command('show', 'get_role_assignment_by_id')
        g.custom_command('delete', 'delete_role_assignment', confirmation=True)

    with self.command_group('synapse role definition', synapse_role_definitions_sdk,
                            custom_command_type=get_custom_sdk('accesscontrol', None)) as g:
        g.custom_command('list', 'list_role_definitions')
        g.custom_show_command('show', 'get_role_definition')

    with self.command_group('synapse role scope', synapse_role_definitions_sdk,
                            custom_command_type=get_custom_sdk('accesscontrol', None)) as g:
        g.custom_command('list', 'list_scopes')

    # Data Plane Commands --Artifacts Linked service operations
    with self.command_group('synapse linked-service', synapse_linked_service_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('create', 'create_or_update_linked_service', supports_no_wait=True)
        g.custom_command('set', 'create_or_update_linked_service', deprecate_info=g.deprecate(redirect='update'), supports_no_wait=True)
        g.custom_command('update', 'create_or_update_linked_service', supports_no_wait=True)
        g.custom_command('list', 'list_linked_service')
        g.custom_show_command('show', 'get_linked_service')
        g.custom_command('delete', 'delete_linked_service', confirmation=True, supports_no_wait=True)

    # Data Plane Commands --Artifacts dataset operations
    with self.command_group('synapse dataset', synapse_dataset_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('create', 'create_or_update_dataset', supports_no_wait=True)
        g.custom_command('set', 'create_or_update_dataset', deprecate_info=g.deprecate(redirect='update'), supports_no_wait=True)
        g.custom_command('update', 'create_or_update_dataset', supports_no_wait=True)
        g.custom_command('list', 'list_datasets')
        g.custom_show_command('show', 'get_dataset')
        g.custom_command('delete', 'delete_dataset', confirmation=True, supports_no_wait=True)

    # Data Plane Commands --Artifacts pipeline operations
    with self.command_group('synapse pipeline', synapse_pipeline_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('create', 'create_or_update_pipeline', supports_no_wait=True)
        g.custom_command('set', 'create_or_update_pipeline', deprecate_info=g.deprecate(redirect='update'), supports_no_wait=True)
        g.custom_command('update', 'create_or_update_pipeline', supports_no_wait=True)
        g.custom_command('list', 'list_pipelines')
        g.custom_show_command('show', 'get_pipeline')
        g.custom_command('delete', 'delete_pipeline', confirmation=True, supports_no_wait=True)
        g.custom_command('create-run', 'create_pipeline_run')

    # Data Plane Commands --Artifacts pipeline run operations
    with self.command_group('synapse pipeline-run', synapse_pipeline_run_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('query-by-workspace', 'query_pipeline_runs_by_workspace')
        g.custom_show_command('show', 'get_pipeline_run')
        g.custom_command('cancel', 'cancel_pipeline_run', confirmation=True)

    with self.command_group('synapse activity-run', synapse_pipeline_run_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('query-by-pipeline-run', 'query_activity_runs')

    # Data Plane Commands --Artifacts trigger operations
    with self.command_group('synapse trigger', synapse_trigger_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('create', 'create_or_update_trigger', supports_no_wait=True)
        g.custom_command('set', 'create_or_update_trigger', deprecate_info=g.deprecate(redirect='update'), supports_no_wait=True)
        g.custom_command('update', 'create_or_update_trigger', supports_no_wait=True)
        g.custom_command('list', 'list_triggers')
        g.custom_show_command('show', 'get_trigger')
        g.custom_command('delete', 'delete_trigger', confirmation=True, supports_no_wait=True)
        g.custom_command('subscribe-to-event', 'subscribe_trigger_to_events', supports_no_wait=True)
        g.custom_command('get-event-subscription-status', 'get_event_subscription_status')
        g.custom_command('unsubscribe-from-event', 'unsubscribe_trigger_from_events', supports_no_wait=True)
        g.custom_command('start', 'start_trigger', supports_no_wait=True)
        g.custom_command('stop', 'stop_trigger', supports_no_wait=True)
        g.custom_wait_command('wait', 'get_trigger')

    # Data Plane Commands --Artifacts trigger run operations
    with self.command_group('synapse trigger-run', synapse_trigger_run_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('rerun', 'rerun_trigger')
        g.custom_command('cancel', 'cancel_trigger')
        g.custom_command('query-by-workspace', 'query_trigger_runs_by_workspace')

    # Data Plane Commands --Artifacts data flow operations
    with self.command_group('synapse data-flow', synapse_data_flow_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('create', 'create_or_update_data_flow', supports_no_wait=True)
        g.custom_command('set', 'create_or_update_data_flow', supports_no_wait=True)
        g.custom_command('list', 'list_data_flows')
        g.custom_show_command('show', 'get_data_flow')
        g.custom_command('delete', 'delete_data_flow', confirmation=True, supports_no_wait=True)

    # Data Plane Commands --Artifacts notebook operations
    with self.command_group('synapse notebook', synapse_notebook_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('create', 'create_or_update_notebook', supports_no_wait=True)
        g.custom_command('set', 'create_or_update_notebook', supports_no_wait=True)
        g.custom_command('import', 'create_or_update_notebook', supports_no_wait=True)
        g.custom_command('list', 'list_notebooks')
        g.custom_show_command('show', 'get_notebook')
        g.custom_command('export', 'export_notebook')
        g.custom_command('delete', 'delete_notebook', confirmation=True, supports_no_wait=True)

    # Data Plane Commands --Artifacts library operations
    with self.command_group('synapse workspace-package', synapse_library_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('upload', 'upload_workspace_package')
        g.custom_command('upload-batch', 'workspace_package_upload_batch')
        g.custom_command('list', 'list_workspace_package')
        g.custom_show_command('show', 'get_workspace_package')
        g.custom_command('delete', 'delete_workspace_package', confirmation=True, supports_no_wait=True)

    # Data Plane Commands --Managed private endpoints operations
    with self.command_group('synapse managed-private-endpoints', synapse_managed_private_endpoints_sdk,
                            custom_command_type=get_custom_sdk('managedprivateendpoints', None)) as g:
        g.custom_show_command('show', 'get_Managed_private_endpoints')
        g.custom_command('create', 'create_Managed_private_endpoints')
        g.custom_command('list', 'list_Managed_private_endpoints')
        g.custom_command('delete', 'delete_Managed_private_endpoints', confirmation=True)

    # Data Plane Commands --Artifacts Spark job definitions operations
    with self.command_group('synapse spark-job-definition', synapse_spark_job_definition_sdk,
                            custom_command_type=get_custom_sdk('artifacts', None)) as g:
        g.custom_command('list', 'list_spark_job_definition')
        g.custom_show_command('show', 'get_spark_job_definition')
        g.custom_command('delete', 'delete_spark_job_definition', supports_no_wait=True)
        g.custom_command('create', 'create_or_update_spark_job_definition', supports_no_wait=True)
        g.custom_wait_command('wait', 'get_spark_job_definition')
        g.custom_command('update', 'create_or_update_spark_job_definition', supports_no_wait=True)

    with self.command_group('synapse', is_preview=True):
        pass

# synapse kusto pool Commands --Managed kusto pool Commands
    with self.command_group('synapse kusto pool',
                            command_type=synapse_kusto_pool_sdk,
                            custom_command_type=get_custom_sdk('kustopool',
                                                               cf_kusto_pool),
                            client_factory=cf_kusto_pool) as g:
        g.custom_command('create', 'synapse_kusto_pool_create', supports_no_wait=True)
        g.custom_command('update', 'synapse_kusto_pool_update', supports_no_wait=True)
        g.custom_command('add-language-extension', 'synapse_kusto_pool_add_language_extension', supports_no_wait=True)
        g.custom_command('detach-follower-database', 'synapse_kusto_pool_detach_follower_database', supports_no_wait=True)
        g.custom_command('remove-language-extension', 'synapse_kusto_pool_remove_language_extension', supports_no_wait=True)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def _compute_client_factory(cli_ctx, **kwargs):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_COMPUTE,
                                   subscription_id=kwargs.get('subscription_id'),
                                   aux_subscriptions=kwargs.get('aux_subscriptions'))


def cf_ni(cli_ctx, _):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    # TODO: Remove hard coded api-version once
    # https://github.com/Azure/azure-rest-api-specs/issues/570
    # is fixed.
    ni = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK).network_interfaces
    ni.api_version = '2016-03-30'
    return ni


def cf_public_ip_addresses(cli_ctx):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    public_ip_ops = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK).public_ip_addresses
    return public_ip_ops


def cf_avail_set(cli_ctx, _):
    return _compute_client_factory(cli_ctx).availability_sets


def cf_vm(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machines


def cf_vm_ext(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_extensions


def cf_vm_ext_image(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_extension_images


def cf_vm_image(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_images


def cf_vm_image_term(cli_ctx, _):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.marketplaceordering import MarketplaceOrderingAgreements
    market_place_client = get_mgmt_service_client(cli_ctx, MarketplaceOrderingAgreements)
    return market_place_client.marketplace_agreements


def cf_usage(cli_ctx, _):
    return _compute_client_factory(cli_ctx).usage


def cf_vmss(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_scale_sets


def cf_vmss_vm(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_scale_set_vms


def cf_vm_sizes(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_sizes


def cf_disks(cli_ctx, _):
    return _compute_client_factory(cli_ctx).disks


def cf_snapshots(cli_ctx, _):
    return _compute_client_factory(cli_ctx).snapshots


def cf_disk_accesses(cli_ctx, _):
    return _compute_client_factory(cli_ctx).disk_accesses


def cf_images(cli_ctx, _):
    return _compute_client_factory(cli_ctx).images


def cf_run_commands(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_run_commands


def cf_vmss_run_commands(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_scale_set_vm_run_commands


def cf_rolling_upgrade_commands(cli_ctx, _):
    return _compute_client_factory(cli_ctx).virtual_machine_scale_set_rolling_upgrades


def cf_galleries(cli_ctx, _):
    return _compute_client_factory(cli_ctx).galleries


def cf_gallery_images(cli_ctx, _):
    return _compute_client_factory(cli_ctx).gallery_images


def cf_gallery_image_versions(cli_ctx, _):
    return _compute_client_factory(cli_ctx).gallery_image_versions


def cf_proximity_placement_groups(cli_ctx, _):
    return _compute_client_factory(cli_ctx).proximity_placement_groups


def cf_dedicated_hosts(cli_ctx, _):
    return _compute_client_factory(cli_ctx).dedicated_hosts


def cf_dedicated_host_groups(cli_ctx, _):
    return _compute_client_factory(cli_ctx).dedicated_host_groups


def _log_analytics_client_factory(cli_ctx, subscription_id, *_):
    from azure.mgmt.loganalytics import LogAnalyticsManagementClient
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, LogAnalyticsManagementClient, subscription_id=subscription_id)


def cf_log_analytics(cli_ctx, subscription_id, *_):
    return _log_analytics_client_factory(cli_ctx, subscription_id)


def cf_log_analytics_data_sources(cli_ctx, subscription_id, *_):
    return _log_analytics_client_factory(cli_ctx, subscription_id).data_sources


def cf_log_analytics_data_plane(cli_ctx, _):
    """Initialize Log Analytics data client for use with CLI."""
    from azure.loganalytics import LogAnalyticsDataClient
    from azure.cli.core._profile import Profile
    profile = Profile(cli_ctx=cli_ctx)
    cred, _, _ = profile.get_login_credentials(
        resource=cli_ctx.cloud.endpoints.log_analytics_resource_id)
    api_version = 'v1'
    return LogAnalyticsDataClient(cred,
                                  base_url=cli_ctx.cloud.endpoints.log_analytics_resource_id + '/' + api_version)


def cf_disk_encryption_set(cli_ctx, _):
    return _compute_client_factory(cli_ctx).disk_encryption_sets


def _dev_test_labs_client_factory(cli_ctx, subscription_id, *_):
    from azure.mgmt.devtestlabs import DevTestLabsClient
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, DevTestLabsClient, subscription_id=subscription_id)


def cf_vm_cl(cli_ctx, *_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.compute import ComputeManagementClient
    return get_mgmt_service_client(cli_ctx,
                                   ComputeManagementClient)


def cf_shared_galleries(cli_ctx, *_):
    return cf_vm_cl(cli_ctx).shared_galleries


def cf_gallery_sharing_profile(cli_ctx, *_):
    return cf_vm_cl(cli_ctx).gallery_sharing_profile


def cf_shared_gallery_image(cli_ctx, *_):
    return cf_vm_cl(cli_ctx).shared_gallery_images


def cf_shared_gallery_image_version(cli_ctx, *_):
    return cf_vm_cl(cli_ctx).shared_gallery_image_versions


def cf_capacity_reservation_groups(cli_ctx, *_):
    return cf_vm_cl(cli_ctx).capacity_reservation_groups


def cf_capacity_reservations(cli_ctx, *_):
    return cf_vm_cl(cli_ctx).capacity_reservations
